package org.gcube.portlets.user.geoportaldataentry.client;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.GeoNaItemRef;
import org.gcube.portlets.user.geoportaldataentry.client.ConstantsGeoPortalDataEntryApp.RECORD_TYPE;
import org.gcube.portlets.user.geoportaldataentry.shared.CommitReport;
import org.gcube.portlets.user.geoportaldataentry.shared.GeoNaFormDataObject;
import org.gcube.portlets.user.geoportaldataentry.shared.GeonaISConfig;
import org.gcube.portlets.user.geoportaldataentry.shared.ResultSetPaginatedData;
import org.gcube.portlets.user.geoportaldataentry.shared.SortFilter;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Aug 4, 2021
 */
@RemoteServiceRelativePath("geoportaldataentryservice")
public interface GeoportalDataEntryService extends RemoteService {

	/**
	 * Save geona data forms.
	 *
	 * @param listGeonaFormObjects the list geona form objects
	 * @return the commit report
	 * @throws Exception the exception
	 */
	CommitReport saveGeonaDataForms(List<GeoNaFormDataObject> listGeonaFormObjects) throws Exception;

	/**
	 * Gets the geona init config.
	 *
	 * @return the geona init config
	 */
	GeonaISConfig getGeonaInitConfig();

	/**
	 * Gets the links for.
	 *
	 * @param itemId     the item id
	 * @param recordType the record type
	 * @return the links for
	 * @throws Exception the exception
	 */
	GeoNaItemRef getLinksFor(String itemId, RECORD_TYPE recordType) throws Exception;

	/**
	 * Gets the list concessioni.
	 *
	 * @param start  the start
	 * @param limit the limit
	 * @param filter the filter
	 * @param reloadFromService the reload from service
	 * @return the list concessioni
	 * @throws Exception the exception
	 */
	public ResultSetPaginatedData getListConcessioni(Integer start, Integer limit, SortFilter filter, boolean reloadFromService) throws Exception;


	/**
	 * Delete record.
	 *
	 * @param itemId the item id
	 * @param recordType the record type
	 * @return true, if successful
	 */
	boolean deleteRecord(String itemId, RECORD_TYPE recordType) throws Exception;
}
